
SERVER_TESTBED_INTERFACE    = "eth0"
WALT_SUBNET                 = "192.168.152.0/24"
DEFAULT_IMAGE               = 'default'
SNMP_TIMEOUT                = 20
DOCKER_HUB_GET_TAGS_URL     = 'https://registry.hub.docker.com/v1/repositories/%(image_name)s/tags'
WALT_DBNAME                 = "walt"
WALT_DBUSER                 = "root"
